@@ -5,34 +5,32 @@ This module provides all necessary functionality to import a model specified
 in the :class:`pysb.core.Model` format.
 """
 
-from .ode_export import (
-    ODEExporter, ODEModel, State, Constant, Parameter, Observable, SigmaY,
-    Expression, LogLikelihood, generate_measurement_symbol
-)
-from .import_utils import (
-    noise_distribution_to_cost_function, _get_str_symbol_identifiers,
-    noise_distribution_to_observable_transformation, _parse_special_functions
-)
-import logging
-from .logging import get_logger, log_execution_time, set_log_level
-
-import sympy as sp
-import numpy as np
 import itertools
+import logging
 import os
 import sys
+from typing import (Any, Callable, Dict, Iterable, List, Optional, Set, Tuple,
+                    Union)
 
-from typing import (
-    List, Union, Dict, Tuple, Set, Iterable, Any, Callable, Optional
-)
+import numpy as np
+import pysb
+import pysb.bng
+import pysb.pattern
+import sympy as sp
+
+from .import_utils import (_get_str_symbol_identifiers,
+                           _parse_special_functions,
+                           generate_measurement_symbol,
+                           noise_distribution_to_cost_function,
+                           noise_distribution_to_observable_transformation,
+                           RESERVED_SYMBOLS)
+from .logging import get_logger, log_execution_time, set_log_level
+from .ode_export import (Constant, Expression, LogLikelihood, ODEExporter,
+                         ODEModel, Observable, Parameter, SigmaY, State)
 
 CL_Prototype = Dict[str, Dict[str, Any]]
 ConservationLaw = Dict[str, Union[str, sp.Basic]]
 
-import pysb.bng
-import pysb
-import pysb.pattern
-
 logger = get_logger(__name__, logging.ERROR)
 
 
@@ -49,6 +47,9 @@ def pysb2amici(
         compute_conservation_laws: bool = True,
         compile: bool = True,
         simplify: Callable = lambda x: sp.powsimp(x, deep=True),
+        # Do not enable by default without testing.
+        # See https://github.com/AMICI-dev/AMICI/pull/1672
+        cache_simplify: bool = False,
         generate_sensitivity_code: bool = True,
 ):
     r"""
@@ -115,6 +116,11 @@ def pysb2amici(
     :param simplify:
         see :attr:`amici.ODEModel._simplify`
 
+    :param cache_simplify:
+            see :func:`amici.ODEModel.__init__`
+            Note that there are possible issues with PySB models:
+            https://github.com/AMICI-dev/AMICI/pull/1672
+
     :param generate_sensitivity_code:
         if set to ``False``, code for sensitivity computation will not be
         generated
@@ -134,6 +140,7 @@ def pysb2amici(
         noise_distributions=noise_distributions,
         compute_conservation_laws=compute_conservation_laws,
         simplify=simplify,
+        cache_simplify=cache_simplify,
         verbose=verbose,
     )
     exporter = ODEExporter(
@@ -160,6 +167,9 @@ def ode_model_from_pysb_importer(
         noise_distributions: Optional[Dict[str, Union[str, Callable]]] = None,
         compute_conservation_laws: bool = True,
         simplify: Callable = sp.powsimp,
+        # Do not enable by default without testing.
+        # See https://github.com/AMICI-dev/AMICI/pull/1672
+        cache_simplify: bool = False,
         verbose: Union[int, bool] = False,
 ) -> ODEModel:
     """
@@ -188,6 +198,11 @@ def ode_model_from_pysb_importer(
     :param simplify:
             see :attr:`amici.ODEModel._simplify`
 
+    :param cache_simplify:
+            see :func:`amici.ODEModel.__init__`
+            Note that there are possible issues with PySB models:
+            https://github.com/AMICI-dev/AMICI/pull/1672
+
     :param verbose: verbosity level for logging, True/False default to
         :attr:`logging.DEBUG`/:attr:`logging.ERROR`
 
@@ -195,7 +210,11 @@ def ode_model_from_pysb_importer(
         New ODEModel instance according to pysbModel
     """
 
-    ode = ODEModel(verbose=verbose, simplify=simplify)
+    ode = ODEModel(
+        verbose=verbose,
+        simplify=simplify,
+        cache_simplify=cache_simplify,
+    )
 
     if constant_parameters is None:
         constant_parameters = []
@@ -325,7 +344,20 @@ def _process_pysb_expressions(
     # they are ordered according to their dependency and we can
     # evaluate them sequentially without reordering. Important to make
     # sure that observables are processed first though.
-    for expr in pysb_model.expressions:
+
+    # we use _constant and _dynamic functions to get access to derived
+    # expressions that are otherwise only accessible as private attribute
+    for expr in pysb_model.expressions_constant(include_derived=True)\
+            | pysb_model.expressions_dynamic(include_derived=True):
+        if any(
+            isinstance(symbol, pysb.Tag)
+            for symbol in expr.expand_expr().free_symbols
+        ):
+            # we only need explicit instantiations of expressions with tags,
+            # which are defined in the derived expressions. The abstract
+            # expressions are not needed and lead to compilation errors so
+            # we skip them.
+            continue
         _add_expression(expr, expr.name, expr.expr,
                         pysb_model, ode_model, observables, sigmas,
                         noise_distributions)
