@@ -234,7 +234,9 @@ void Model::initializeStates(AmiVector &x) {
         fx0(x);
     } else {
         std::vector<realtype> x0_solver(nx_solver, 0.0);
-        ftotal_cl(state_.total_cl.data(), x0data_.data());
+        ftotal_cl(state_.total_cl.data(), x0data_.data(),
+                  state_.unscaledParameters.data(),
+                  state_.fixedParameters.data());
         fx_solver(x0_solver.data(), x0data_.data());
         std::copy(x0_solver.cbegin(), x0_solver.cend(), x.data());
     }
@@ -250,7 +252,11 @@ void Model::initializeStateSensitivities(AmiVectorArray &sx,
         for (int ip = 0; ip < nplist(); ip++) {
             if (ncl() > 0)
                 stcl = &state_.stotal_cl.at(plist(ip) * ncl());
-            fstotal_cl(stcl, &sx0data_.at(ip * nx_rdata), plist(ip));
+            fstotal_cl(stcl, &sx0data_.at(ip * nx_rdata), plist(ip),
+                       derived_state_.x_rdata_.data(),
+                       state_.unscaledParameters.data(),
+                       state_.fixedParameters.data(),
+                       state_.total_cl.data());
             fsx_solver(sx0_solver_slice.data(), &sx0data_.at(ip * nx_rdata));
             for (int ix = 0; ix < nx_solver; ix++) {
                 sx.at(ix, ip) = sx0_solver_slice.at(ix);
@@ -843,9 +849,29 @@ void Model::getObservableSigma(gsl::span<realtype> sigmay, const int it,
 }
 
 void Model::getObservableSigmaSensitivity(gsl::span<realtype> ssigmay,
+                                          gsl::span<const realtype> sy,
                                           const int it, const ExpData *edata) {
     fdsigmaydp(it, edata);
     writeSlice(derived_state_.dsigmaydp_, ssigmay);
+
+    if(pythonGenerated) {
+        // ssigmay = dsigmaydy*(dydx_solver*sx+dydp)+dsigmaydp
+        //         = dsigmaydy*sy+dsigmaydp
+
+        fdsigmaydy(it, edata);
+
+        // compute ssigmay = 1.0 * dsigmaydp + 1.0 * dsigmaydy * sy
+        // dsigmaydp C[ny,nplist] += dsigmaydy A[ny,ny] * sy B[ny,nplist]
+        //             M  N                      M  K          K  N
+        //             ldc                       lda           ldb
+        amici_dgemm(BLASLayout::colMajor, BLASTranspose::noTrans,
+                    BLASTranspose::noTrans, ny, nplist(), ny, 1.0,
+                    derived_state_.dsigmaydy_.data(), ny,
+                    sy.data(), ny, 1.0, ssigmay.data(), ny);
+    }
+
+    if (always_check_finite_)
+        checkFinite(ssigmay, "ssigmay");
 }
 
 void Model::addObservableObjective(realtype &Jy, const int it,
@@ -1076,7 +1102,7 @@ void Model::getEventTimeSensitivity(std::vector<realtype> &stau,
     for (int ip = 0; ip < nplist(); ip++) {
         fstau(&stau.at(ip), t, computeX_pos(x),
               state_.unscaledParameters.data(), state_.fixedParameters.data(),
-              state_.h.data(), sx.data(ip),
+              state_.h.data(), state_.total_cl.data(), sx.data(ip),
               plist(ip), ie);
     }
 }
@@ -1084,9 +1110,9 @@ void Model::getEventTimeSensitivity(std::vector<realtype> &stau,
 void Model::addStateEventUpdate(AmiVector &x, const int ie, const realtype t,
                                 const AmiVector &xdot,
                                 const AmiVector &xdot_old) {
-    
+
     derived_state_.deltax_.assign(nx_solver, 0.0);
-    
+
     std::copy_n(computeX_pos(x), nx_solver, x.data());
 
     // compute update
@@ -1120,7 +1146,8 @@ void Model::addStateSensitivityEventUpdate(AmiVectorArray &sx, const int ie,
                  state_.unscaledParameters.data(),
                  state_.fixedParameters.data(),
                  state_.h.data(), derived_state_.w_.data(), plist(ip), ie,
-                 xdot.data(), xdot_old.data(), sx.data(ip), &stau.at(ip));
+                 xdot.data(), xdot_old.data(), sx.data(ip), &stau.at(ip),
+                 state_.total_cl.data());
 
         if (always_check_finite_) {
             app->checkFinite(derived_state_.deltasx_, "deltasx");
@@ -1214,7 +1241,9 @@ void Model::fx0(AmiVector &x) {
         state_.unscaledParameters.data(),
         state_.fixedParameters.data());
     fx_solver(x.data(), derived_state_.x_rdata_.data());
-    ftotal_cl(state_.total_cl.data(), derived_state_.x_rdata_.data());
+    ftotal_cl(state_.total_cl.data(), derived_state_.x_rdata_.data(),
+              state_.unscaledParameters.data(),
+              state_.fixedParameters.data());
 
     if (always_check_finite_) {
         checkFinite(derived_state_.x_rdata_, "x0 x_rdata");
@@ -1225,13 +1254,14 @@ void Model::fx0(AmiVector &x) {
 void Model::fx0_fixedParameters(AmiVector &x) {
     if (!getReinitializeFixedParameterInitialStates())
         return;
-    
+
     /* we transform to the unreduced states x_rdata and then apply
      x0_fixedparameters to (i) enable updates to states that were removed from
      conservation laws and (ii) be able to correctly compute total abundances
      after updating the state variables */
     fx_rdata(derived_state_.x_rdata_.data(), computeX_pos(x),
-             state_.total_cl.data());
+             state_.total_cl.data(), state_.unscaledParameters.data(),
+             state_.fixedParameters.data());
     fx0_fixedParameters(derived_state_.x_rdata_.data(),
                         simulation_parameters_.tstart_,
                         state_.unscaledParameters.data(),
@@ -1240,7 +1270,9 @@ void Model::fx0_fixedParameters(AmiVector &x) {
                         );
     fx_solver(x.data(), derived_state_.x_rdata_.data());
     /* update total abundances */
-    ftotal_cl(state_.total_cl.data(), derived_state_.x_rdata_.data());
+    ftotal_cl(state_.total_cl.data(), derived_state_.x_rdata_.data(),
+              state_.unscaledParameters.data(),
+              state_.fixedParameters.data());
 }
 
 void Model::fsx0(AmiVectorArray &sx, const AmiVector &x) {
@@ -1255,7 +1287,11 @@ void Model::fsx0(AmiVectorArray &sx, const AmiVector &x) {
              computeX_pos(x), state_.unscaledParameters.data(),
              state_.fixedParameters.data(), plist(ip));
         fsx_solver(sx.data(ip), derived_state_.sx_rdata_.data());
-        fstotal_cl(stcl, derived_state_.sx_rdata_.data(), plist(ip));
+        fstotal_cl(stcl, derived_state_.sx_rdata_.data(), plist(ip),
+                   derived_state_.x_rdata_.data(),
+                   state_.unscaledParameters.data(),
+                   state_.fixedParameters.data(),
+                   state_.total_cl.data());
     }
 }
 
@@ -1266,7 +1302,11 @@ void Model::fsx0_fixedParameters(AmiVectorArray &sx, const AmiVector &x) {
     for (int ip = 0; ip < nplist(); ip++) {
         if (ncl() > 0)
             stcl = &state_.stotal_cl.at(plist(ip) * ncl());
-        fsx_rdata(derived_state_.sx_rdata_.data(), sx.data(ip), stcl, plist(ip));
+        fsx_rdata(derived_state_.sx_rdata_.data(), sx.data(ip), stcl,
+                  state_.unscaledParameters.data(),
+                  state_.fixedParameters.data(),
+                  x.data(), state_.total_cl.data(),
+                  plist(ip));
         fsx0_fixedParameters(derived_state_.sx_rdata_.data(),
                              simulation_parameters_.tstart_,
                              computeX_pos(x),
@@ -1275,24 +1315,33 @@ void Model::fsx0_fixedParameters(AmiVectorArray &sx, const AmiVector &x) {
                              plist(ip),
                              simulation_parameters_.reinitialization_state_idxs_sim);
         fsx_solver(sx.data(ip), derived_state_.sx_rdata_.data());
-        fstotal_cl(stcl, derived_state_.sx_rdata_.data(), plist(ip));
+        fstotal_cl(stcl, derived_state_.sx_rdata_.data(), plist(ip),
+                   derived_state_.x_rdata_.data(),
+                   state_.unscaledParameters.data(),
+                   state_.fixedParameters.data(),
+                   state_.total_cl.data());
     }
 }
 
 void Model::fsdx0() {}
 
 void Model::fx_rdata(AmiVector &x_rdata, const AmiVector &x) {
-    fx_rdata(x_rdata.data(), computeX_pos(x), state_.total_cl.data());
+    fx_rdata(x_rdata.data(), computeX_pos(x), state_.total_cl.data(),
+             state_.unscaledParameters.data(), state_.fixedParameters.data());
     if (always_check_finite_)
         checkFinite(x_rdata.getVector(), "x_rdata");
 }
 
-void Model::fsx_rdata(AmiVectorArray &sx_rdata, const AmiVectorArray &sx) {
+void Model::fsx_rdata(AmiVectorArray &sx_rdata, const AmiVectorArray &sx,
+                      AmiVector const& x_solver) {
     realtype *stcl = nullptr;
     for (int ip = 0; ip < nplist(); ip++) {
         if (ncl() > 0)
             stcl = &state_.stotal_cl.at(plist(ip) * ncl());
-        fsx_rdata(sx_rdata.data(ip), sx.data(ip), stcl, ip);
+        fsx_rdata(sx_rdata.data(ip), sx.data(ip), stcl,
+                  state_.unscaledParameters.data(),
+                  state_.fixedParameters.data(), x_solver.data(),
+                  state_.total_cl.data(), plist(ip));
     }
 }
 
@@ -1347,7 +1396,7 @@ void Model::initializeVectors() {
 void Model::fy(const realtype t, const AmiVector &x) {
     if (!ny)
         return;
-    
+
     auto x_pos = computeX_pos(x);
 
     derived_state_.y_.assign(ny, 0.0);
@@ -1365,7 +1414,7 @@ void Model::fy(const realtype t, const AmiVector &x) {
 void Model::fdydp(const realtype t, const AmiVector &x) {
     if (!ny)
         return;
-    
+
     auto x_pos = computeX_pos(x);
 
     derived_state_.dydp_.assign(ny * nplist(), 0.0);
@@ -1378,7 +1427,8 @@ void Model::fdydp(const realtype t, const AmiVector &x) {
             fdydp(&derived_state_.dydp_.at(ip * ny), t, x_pos,
                   state_.unscaledParameters.data(),
                   state_.fixedParameters.data(), state_.h.data(), plist(ip),
-                  derived_state_.w_.data(), state_.stotal_cl.data());
+                  derived_state_.w_.data(), state_.total_cl.data(),
+                  state_.stotal_cl.data());
         } else {
             fdydp(&derived_state_.dydp_.at(ip * ny), t, x_pos,
                   state_.unscaledParameters.data(),
@@ -1394,7 +1444,7 @@ void Model::fdydp(const realtype t, const AmiVector &x) {
 void Model::fdydx(const realtype t, const AmiVector &x) {
     if (!ny)
         return;
-    
+
     auto x_pos = computeX_pos(x);
 
     derived_state_.dydx_.assign(ny * nx_solver, 0.0);
@@ -1417,8 +1467,11 @@ void Model::fsigmay(const int it, const ExpData *edata) {
 
     derived_state_.sigmay_.assign(ny, 0.0);
 
-    fsigmay(derived_state_.sigmay_.data(), getTimepoint(it), state_.unscaledParameters.data(),
-            state_.fixedParameters.data());
+    fsigmay(derived_state_.sigmay_.data(),
+            getTimepoint(it),
+            state_.unscaledParameters.data(),
+            state_.fixedParameters.data(),
+            derived_state_.y_.data());
 
     if (edata) {
         auto sigmay_edata = edata->getObservedDataStdDevPtr(it);
@@ -1451,6 +1504,7 @@ void Model::fdsigmaydp(const int it, const ExpData *edata) {
         fdsigmaydp(&derived_state_.dsigmaydp_.at(ip * ny), getTimepoint(it),
                    state_.unscaledParameters.data(),
                    state_.fixedParameters.data(),
+                   derived_state_.y_.data(),
                    plist(ip));
 
     // sigmas in edata override model-sigma -> for those sigmas, set dsigmaydp
@@ -1470,6 +1524,36 @@ void Model::fdsigmaydp(const int it, const ExpData *edata) {
     }
 }
 
+void Model::fdsigmaydy(const int it, const ExpData *edata) {
+    if (!ny)
+        return;
+
+    derived_state_.dsigmaydy_.assign(ny * ny, 0.0);
+
+    // get dsigmaydy slice (ny) for current timepoint
+    fdsigmaydy(derived_state_.dsigmaydy_.data(), getTimepoint(it),
+               state_.unscaledParameters.data(),
+               state_.fixedParameters.data(),
+               derived_state_.y_.data());
+
+   // sigmas in edata override model-sigma -> for those sigmas, set dsigmaydy
+   // to zero
+    if (edata) {
+        for (int isigmay = 0; isigmay < nytrue; ++isigmay) {
+            if (!edata->isSetObservedDataStdDev(it, isigmay))
+                continue;
+            for (int iy = 0; iy < nytrue; ++iy) {
+                derived_state_.dsigmaydy_.at(isigmay * ny + iy) = 0.0;
+            }
+        }
+    }
+
+    if (always_check_finite_) {
+        app->checkFinite(derived_state_.dsigmaydy_, "dsigmaydy");
+    }
+}
+
+
 void Model::fdJydy(const int it, const AmiVector &x, const ExpData &edata) {
     if (!ny)
         return;
@@ -1478,6 +1562,10 @@ void Model::fdJydy(const int it, const AmiVector &x, const ExpData &edata) {
     fsigmay(it, &edata);
 
     if (pythonGenerated) {
+        fdJydsigma(it, x, edata);
+        fdsigmaydy(it, &edata);
+        SUNMatrixWrapper tmp_dense(nJ, ny);
+
         for (int iyt = 0; iyt < nytrue; iyt++) {
             if (!derived_state_.dJydy_.at(iyt).capacity())
                 continue;
@@ -1494,6 +1582,25 @@ void Model::fdJydy(const int it, const AmiVector &x, const ExpData &edata) {
                    derived_state_.sigmay_.data(),
                    edata.getObservedDataPtr(it));
 
+            // dJydy += dJydsigma * dsigmaydy
+            // C(nJ,ny)  A(nJ,ny)  * B(ny,ny)
+            // sparse    dense       dense
+            tmp_dense.zero();
+            amici_dgemm(BLASLayout::colMajor, BLASTranspose::noTrans,
+                        BLASTranspose::noTrans, nJ, ny, ny, 1.0,
+                        &derived_state_.dJydsigma_.at(iyt * nJ * ny), nJ,
+                        derived_state_.dsigmaydy_.data(), ny, 1.0,
+                        tmp_dense.data(), nJ);
+
+            auto tmp_sparse = SUNMatrixWrapper(tmp_dense, 0.0, CSC_MAT);
+            auto ret = SUNMatScaleAdd(1.0, derived_state_.dJydy_.at(iyt).get(),
+                                      tmp_sparse.get());
+            if(ret != SUNMAT_SUCCESS) {
+                throw AmiException("SUNMatScaleAdd failed with status %d in %s",
+                                   ret, __func__);
+            }
+            derived_state_.dJydy_.at(iyt).refresh();
+
             if (always_check_finite_) {
                 app->checkFinite(
                             gsl::make_span(derived_state_.dJydy_.at(iyt).get()),
@@ -1547,7 +1654,6 @@ void Model::fdJydp(const int it, const AmiVector &x, const ExpData &edata) {
     // dJydy         nJ, nytrue x ny
     // dydp          nplist * ny
     // dJydp         nplist x nJ
-    // dJydsigma
     if (!ny)
         return;
 
@@ -2067,7 +2173,8 @@ void Model::fdwdw(const realtype t, const realtype *x) {
 }
 
 void Model::fx_rdata(realtype *x_rdata, const realtype *x_solver,
-                     const realtype * /*tcl*/) {
+                     const realtype * /*tcl*/, const realtype */*p*/,
+                     const realtype */*k*/) {
     if (nx_solver != nx_rdata)
         throw AmiException(
             "A model that has differing nx_solver and nx_rdata needs "
@@ -2076,8 +2183,43 @@ void Model::fx_rdata(realtype *x_rdata, const realtype *x_solver,
 }
 
 void Model::fsx_rdata(realtype *sx_rdata, const realtype *sx_solver,
-                      const realtype *stcl, const int /*ip*/) {
-    fx_rdata(sx_rdata, sx_solver, stcl);
+                      const realtype *stcl, const realtype *p,
+                      const realtype *k, const realtype *x_solver,
+                      const realtype *tcl,
+                      const int ip) {
+    if (nx_solver == nx_rdata) {
+        std::copy_n(sx_solver, nx_solver, sx_rdata);
+        return;
+    }
+
+    // sx_rdata = dx_rdata/dx_solver * sx_solver
+    //             + dx_rdata/d_tcl * stcl + dxrdata/dp
+
+    // 1) sx_rdata(nx_rdata, 1) = dx_rdatadp
+    std::fill_n(sx_rdata, nx_rdata, 0.0);
+    fdx_rdatadp(sx_rdata, x_solver, tcl, p, k, ip);
+
+
+    // the following could be moved to the calling function, as it's independent
+    //  of `ip`
+
+    // 2) sx_rdata(nx_rdata, 1) +=
+    //          dx_rdata/dx_solver(nx_rdata,nx_solver) * sx_solver(nx_solver, 1)
+    derived_state_.dx_rdatadx_solver.zero();
+    fdx_rdatadx_solver(derived_state_.dx_rdatadx_solver.data(),
+                       x_solver, tcl, p, k);
+    fdx_rdatadx_solver_colptrs(derived_state_.dx_rdatadx_solver);
+    fdx_rdatadx_solver_rowvals(derived_state_.dx_rdatadx_solver);
+    derived_state_.dx_rdatadx_solver.multiply(gsl::make_span(sx_rdata, nx_rdata),
+                                              gsl::make_span(sx_solver, nx_solver));
+
+    // 3) sx_rdata(nx_rdata, 1) += dx_rdata/d_tcl(nx_rdata,ntcl) * stcl
+    derived_state_.dx_rdatadtcl.zero();
+    fdx_rdatadtcl(derived_state_.dx_rdatadtcl.data(), x_solver, tcl, p, k);
+    fdx_rdatadtcl_colptrs(derived_state_.dx_rdatadtcl);
+    fdx_rdatadtcl_rowvals(derived_state_.dx_rdatadtcl);
+    derived_state_.dx_rdatadtcl.multiply(gsl::make_span(sx_rdata, nx_rdata),
+                                         gsl::make_span(stcl, (nx_rdata - nx_solver)));
 }
 
 void Model::fx_solver(realtype *x_solver, const realtype *x_rdata) {
@@ -2095,7 +2237,8 @@ void Model::fsx_solver(realtype *sx_solver, const realtype *sx_rdata) {
     fx_solver(sx_solver, sx_rdata);
 }
 
-void Model::ftotal_cl(realtype * /*total_cl*/, const realtype * /*x_rdata*/) {
+void Model::ftotal_cl(realtype * /*total_cl*/, const realtype * /*x_rdata*/,
+                      const realtype */*p*/, const realtype */*k*/) {
     if (nx_solver != nx_rdata)
         throw AmiException(
             "A model that has differing nx_solver and nx_rdata needs "
@@ -2103,11 +2246,30 @@ void Model::ftotal_cl(realtype * /*total_cl*/, const realtype * /*x_rdata*/) {
 }
 
 void Model::fstotal_cl(realtype *stotal_cl, const realtype *sx_rdata,
-                       const int /*ip*/) {
-    /* for the moment we do not need an implementation of fstotal_cl as
-     * we can simply reuse ftotal_cl and replace states by their
-     * sensitivities */
-    ftotal_cl(stotal_cl, sx_rdata);
+                       const int ip, const realtype *x_rdata,
+                       const realtype *p, const realtype *k,
+                       const realtype *tcl) {
+    if (nx_solver == nx_rdata)
+        return;
+
+    // stotal_cl(ncl,1) =
+    //            dtotal_cl/dp(ncl,1)
+    //          + dtotal_cl/dx_rdata(ncl,nx_rdata) * sx_rdata(nx_rdata,1)
+
+    // 1) stotal_cl = dtotal_cl/dp
+    std::fill_n(stotal_cl, ncl(), 0.0);
+    fdtotal_cldp(stotal_cl, x_rdata, p, k, ip);
+
+
+    // 2) stotal_cl += dtotal_cl/dx_rdata(ncl,nx_rdata) * sx_rdata(nx_rdata,1)
+    derived_state_.dtotal_cldx_rdata.zero();
+    fdtotal_cldx_rdata(derived_state_.dtotal_cldx_rdata.data(),
+                       x_rdata, tcl, p, k);
+    fdtotal_cldx_rdata_colptrs(derived_state_.dtotal_cldx_rdata);
+    fdtotal_cldx_rdata_rowvals(derived_state_.dtotal_cldx_rdata);
+    derived_state_.dtotal_cldx_rdata.multiply(
+        gsl::make_span(stotal_cl, ncl()),
+        gsl::make_span(sx_rdata, nx_rdata));
 }
 
 const_N_Vector Model::computeX_pos(const_N_Vector x) {
