@@ -56,6 +56,7 @@ TPL_DYDX_DEF
 TPL_DYDP_DEF
 TPL_SIGMAY_DEF
 TPL_DSIGMAYDP_DEF
+TPL_DSIGMAYDY_DEF
 TPL_W_DEF
 TPL_X0_DEF
 TPL_X0_FIXEDPARAMETERS_DEF
@@ -69,7 +70,17 @@ TPL_DELTASX_DEF
 TPL_X_RDATA_DEF
 TPL_X_SOLVER_DEF
 TPL_TOTAL_CL_DEF
-
+TPL_DX_RDATADX_SOLVER_DEF
+TPL_DX_RDATADX_SOLVER_COLPTRS_DEF
+TPL_DX_RDATADX_SOLVER_ROWVALS_DEF
+TPL_DX_RDATADP_DEF
+TPL_DX_RDATADTCL_DEF
+TPL_DX_RDATADTCL_COLPTRS_DEF
+TPL_DX_RDATADTCL_ROWVALS_DEF
+TPL_DTOTAL_CLDP_DEF
+TPL_DTOTAL_CLDX_RDATA_DEF
+TPL_DTOTAL_CLDX_RDATA_COLPTRS_DEF
+TPL_DTOTAL_CLDX_RDATA_ROWVALS_DEF
 /**
  * @brief AMICI-generated model subclass.
  */
@@ -100,6 +111,9 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
                   TPL_NDWDW,                               // ndwdw
                   TPL_NDXDOTDW,                            // ndxdotdw
                   TPL_NDJYDY,                              // ndjydy
+                  TPL_NDXRDATADXSOLVER,                    // ndxrdatadxsolver
+                  TPL_NDXRDATADTCL,                        // ndxrdatadtcl
+                  TPL_NDTOTALCLDXRDATA,                        // ndtotal_cldx_rdata
                   0,                                       // nnz
                   TPL_UBW,                                 // ubw
                   TPL_LBW                                  // lbw
@@ -291,6 +305,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
 
     TPL_DSIGMAYDP_IMPL
 
+    TPL_DSIGMAYDY_IMPL
+
     /**
      * @brief model specific implementation of fsigmaz
      * @param dsigmazdp partial derivative of standard deviation of event
@@ -464,6 +480,22 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
 
     TPL_TOTAL_CL_IMPL
 
+    TPL_DX_RDATADX_SOLVER_IMPL
+    TPL_DX_RDATADX_SOLVER_COLPTRS_IMPL
+    TPL_DX_RDATADX_SOLVER_ROWVALS_IMPL
+
+    TPL_DX_RDATADP_IMPL
+
+    TPL_DX_RDATADTCL_IMPL
+    TPL_DX_RDATADTCL_COLPTRS_IMPL
+    TPL_DX_RDATADTCL_ROWVALS_IMPL
+
+    TPL_DTOTAL_CLDP_IMPL
+
+    TPL_DTOTAL_CLDX_RDATA_IMPL
+    TPL_DTOTAL_CLDX_RDATA_COLPTRS_IMPL
+    TPL_DTOTAL_CLDX_RDATA_ROWVALS_IMPL
+
     std::string getName() const override {
         return "TPL_MODELNAME";
     }
